@@ -9,8 +9,8 @@ from .utils import (
     _get_nice_quiver_magnitude,
     _infer_xy_labels,
     _process_cmap_cbar_kwargs,
-    import_matplotlib_pyplot,
     label_from_attrs,
+    plt,
 )
 
 # Overrides axes.labelsize, xtick.major.size, ytick.major.size
@@ -35,13 +35,13 @@ def _nicetitle(coord, value, maxchar, template):
 
 class FacetGrid:
     """
-    Initialize the matplotlib figure and FacetGrid object.
+    Initialize the Matplotlib figure and FacetGrid object.
 
     The :class:`FacetGrid` is an object that links a xarray DataArray to
-    a matplotlib figure with a particular structure.
+    a Matplotlib figure with a particular structure.
 
     In particular, :class:`FacetGrid` is used to draw plots with multiple
-    Axes where each Axes shows the same relationship conditioned on
+    axes, where each axes shows the same relationship conditioned on
     different levels of some dimension. It's possible to condition on up to
     two variables by assigning variables to the rows and columns of the
     grid.
@@ -59,19 +59,19 @@ class FacetGrid:
 
     Attributes
     ----------
-    axes : numpy object array
-        Contains axes in corresponding position, as returned from
-        plt.subplots
-    col_labels : list
-        list of :class:`matplotlib.text.Text` instances corresponding to column titles.
-    row_labels : list
-        list of :class:`matplotlib.text.Text` instances corresponding to row titles.
-    fig : matplotlib.Figure
-        The figure containing all the axes
-    name_dicts : numpy object array
-        Contains dictionaries mapping coordinate names to values. None is
-        used as a sentinel value for axes which should remain empty, ie.
-        sometimes the bottom right grid
+    axes : ndarray of matplotlib.axes.Axes
+        Array containing axes in corresponding position, as returned from
+        :py:func:`matplotlib.pyplot.subplots`.
+    col_labels : list of matplotlib.text.Text
+        Column titles.
+    row_labels : list of matplotlib.text.Text
+        Row titles.
+    fig : matplotlib.figure.Figure
+        The figure containing all the axes.
+    name_dicts : ndarray of dict
+        Array containing dictionaries mapping coordinate names to values. ``None`` is
+        used as a sentinel value for axes that should remain empty, i.e.,
+        sometimes the rightmost grid positions in the bottom row.
     """
 
     def __init__(
@@ -91,31 +91,31 @@ class FacetGrid:
         Parameters
         ----------
         data : DataArray
-            xarray DataArray to be plotted
-        row, col : strings
-            Dimesion names that define subsets of the data, which will be drawn
+            xarray DataArray to be plotted.
+        row, col : str
+            Dimension names that define subsets of the data, which will be drawn
             on separate facets in the grid.
         col_wrap : int, optional
-            "Wrap" the column variable at this width, so that the column facets
+            "Wrap" the grid the for the column variable after this number of columns,
+            adding rows if ``col_wrap`` is less than the number of facets.
         sharex : bool, optional
-            If true, the facets will share x axes
+            If true, the facets will share *x* axes.
         sharey : bool, optional
-            If true, the facets will share y axes
+            If true, the facets will share *y* axes.
         figsize : tuple, optional
             A tuple (width, height) of the figure in inches.
             If set, overrides ``size`` and ``aspect``.
         aspect : scalar, optional
             Aspect ratio of each facet, so that ``aspect * size`` gives the
-            width of each facet in inches
+            width of each facet in inches.
         size : scalar, optional
-            Height (in inches) of each facet. See also: ``aspect``
+            Height (in inches) of each facet. See also: ``aspect``.
         subplot_kws : dict, optional
-            Dictionary of keyword arguments for matplotlib subplots
+            Dictionary of keyword arguments for Matplotlib subplots
+            (:py:func:`matplotlib.pyplot.subplots`).
 
         """
 
-        plt = import_matplotlib_pyplot()
-
         # Handle corner case of nonunique coordinates
         rep_col = col is not None and not data[col].to_index().is_unique
         rep_row = row is not None and not data[row].to_index().is_unique
@@ -173,11 +173,11 @@ class FacetGrid:
         )
 
         # Set up the lists of names for the row and column facet variables
-        col_names = list(data[col].values) if col else []
-        row_names = list(data[row].values) if row else []
+        col_names = list(data[col].to_numpy()) if col else []
+        row_names = list(data[row].to_numpy()) if row else []
 
         if single_group:
-            full = [{single_group: x} for x in data[single_group].values]
+            full = [{single_group: x} for x in data[single_group].to_numpy()]
             empty = [None for x in range(nrow * ncol - len(full))]
             name_dicts = full + empty
         else:
@@ -251,7 +251,7 @@ class FacetGrid:
             raise ValueError("cbar_ax not supported by FacetGrid.")
 
         cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
-            func, self.data.values, **kwargs
+            func, self.data.to_numpy(), **kwargs
         )
 
         self._cmap_extend = cmap_params.get("extend")
@@ -347,7 +347,7 @@ class FacetGrid:
 
         if hue and meta_data["hue_style"] == "continuous":
             cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
-                func, self.data[hue].values, **kwargs
+                func, self.data[hue].to_numpy(), **kwargs
             )
             kwargs["meta_data"]["cmap_params"] = cmap_params
             kwargs["meta_data"]["cbar_kwargs"] = cbar_kwargs
@@ -423,7 +423,7 @@ class FacetGrid:
     def add_legend(self, **kwargs):
         self.figlegend = self.fig.legend(
             handles=self._mappables[-1],
-            labels=list(self._hue_var.values),
+            labels=list(self._hue_var.to_numpy()),
             title=self._hue_label,
             loc="center right",
             **kwargs,
@@ -431,7 +431,7 @@ class FacetGrid:
         self._adjust_fig_for_guide(self.figlegend)
 
     def add_colorbar(self, **kwargs):
-        """Draw a colorbar"""
+        """Draw a colorbar."""
         kwargs = kwargs.copy()
         if self._cmap_extend is not None:
             kwargs.setdefault("extend", self._cmap_extend)
@@ -517,10 +517,8 @@ class FacetGrid:
         self: FacetGrid object
 
         """
-        import matplotlib as mpl
-
         if size is None:
-            size = mpl.rcParams["axes.labelsize"]
+            size = plt.rcParams["axes.labelsize"]
 
         nicetitle = functools.partial(_nicetitle, maxchar=maxchar, template=template)
 
@@ -564,7 +562,7 @@ class FacetGrid:
 
     def set_ticks(self, max_xticks=_NTICKS, max_yticks=_NTICKS, fontsize=_FONTSIZE):
         """
-        Set and control tick behavior
+        Set and control tick behavior.
 
         Parameters
         ----------
@@ -617,13 +615,11 @@ class FacetGrid:
         self : FacetGrid object
 
         """
-        plt = import_matplotlib_pyplot()
-
         for ax, namedict in zip(self.axes.flat, self.name_dicts.flat):
             if namedict is not None:
                 data = self.data.loc[namedict]
                 plt.sca(ax)
-                innerargs = [data[a].values for a in args]
+                innerargs = [data[a].to_numpy() for a in args]
                 maybe_mappable = func(*innerargs, **kwargs)
                 # TODO: better way to verify that an artist is mappable?
                 # https://stackoverflow.com/questions/33023036/is-it-possible-to-detect-if-a-matplotlib-artist-is-a-mappable-suitable-for-use-w#33023522
